<?php
/**
  * @file
  * Defines Drupal\suggestedterms\Suggestions.
  * Queries for vocabulary terms.
  */
namespace Drupal\suggestedterms\Suggestions;

class Suggestions {
  protected $sort;
  protected $type;
  protected $max;
  protected $vocab;

  public function __construct($sort, $type, $max, $vocab) {
    $this->sort = $sort;
    $this->type = $type;
    $this->max = $max;
    $this->vocab = $vocab;
  }

  /**
   * Takes a query and returns a render array for a list of links.
   *
   * @param obj $query
   *  The query that will yield the listed items.
   * @return array
   *  A render array for an item list.
   */
  public function buildRenderArray($query) {
    // Create an item list render array to hold our terms.
    $terms = array(
      '#attributes' => array(
        'class'  => 'suggestedterms',
        ),
      '#theme' => 'item_list',
      '#type' => 'ul',
    );
    // Get the term names out from the query results and add classes.
    foreach ($query->execute() as $record) {
      $query_result[] = $record->name;
      $term = strip_tags($record->name);

      // Add classes.
      $attributes = array(
        'class' => array(
          'suggestedterm'
        ),
      );
      // Format each term as a link with no destination.
      $variables = l($term, NULL, array('attributes' => $attributes));

      // Add the link to our list render array.
      $terms['#items'][] = $variables;
      // Reset $selected for next one.
    }
    return $terms;
  }

  /**
   * Creates a query for the list of terms.
   *
   * @return obj
   *  A query for terms.
   */
  public function buildTermQuery() {
    // Create a query for term names by vocabulary.
    $query = db_select('taxonomy_term_field_data', 't');
    $query->fields('t', array('name'));
    $query->condition('t.vid', $this->vocab, '=');

    // Get sort order set by admin.
    switch ($this->sort) {
      case SUGGESTEDTERMS_SORT_POPULAR:
        $this->sortPopular($query, $this->type);
        break;
      case SUGGESTEDTERMS_SORT_RECENT:
        $this->sortRecent($query, $this->type);
        break;
      case SUGGESTEDTERMS_SORT_WEIGHT:
        $this->sortWeight($query, $this->type);
        break;
      case SUGGESTEDTERMS_SORT_USED:
        $this->sortUsed($query, $this->type);
        break;
      default:
        $this->sortName($query, $this->type);
        break;
    }
    // Limit the results to the number set in config.
    if ($this->max >= 1) {
      $query->range(0, $this->max);
    }
    return $query;
  }

  /**
   * Sorts a query of terms by usage number.
   *
   * @param obj $query
   *  The query object to modify
   */
  protected function sortPopular(&$query) {
    if ($this->type == ' INNER ') {
      $query->innerJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
    }
    elseif ($this->type = ' LEFT ') {
      $query->leftJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
    }
    else {
      watchdog('Suggested Terms' , 'Join type should be LEFT or INNER, but something else got passed instead.', WATCHDOG_ERROR);
    }
    // Count the number of times the term's used and sort by that.
    $query->addExpression('COUNT(t.name)', 'total');
    $query->groupBy('t.name');
    $query->orderBy('total', 'DESC');
  }

  /**
   * Sorts a query of terms by date created.
   *
   * @param obj $query
   *  The query object to modify
   */
  protected function sortRecent(&$query) {
    // List all the terms in the specified vocabulary and sort by term ID (We assume larger numbers were added more recently).
    $query->groupBy('t.name');
    $query->orderBy('t.tid', 'DESC');
    $query->condition('t.vid', $this->vocab, '=');
    if ($this->type == ' INNER ') {
      $query->innerJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
    }
    elseif ($this->type = ' LEFT ') {
      $query->leftJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
    }
    else {
      watchdog('Suggested Terms' , 'Join type should be LEFT or INNER, but something else got passed instead.', WATCHDOG_ERROR);
    }
  }


  /**
   * Sorts a query of terms by term weight.
   *
   * @param obj $query
   *  The query object to modify
   */
  protected function sortWeight(&$query) {
    // List all the terms in the specified vocabulary and sort by term weight.
    if ($this->type == ' INNER ') {
      $query->innerJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
    }
    elseif ($this->type = ' LEFT ') {
      $query->leftJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
    }
    else {
      watchdog('Suggested Terms' , 'Join type should be LEFT or INNER, but something else got passed instead.', WATCHDOG_ERROR);
    }
    $query->groupBy('name');
    $query->orderBy('weight', 'ASC');
    $query->orderBy('name');
  }


  /**
   * Sorts a query of terms by most recent usage.
   *
   * @param obj $query
   *  The query object to modify
   */
  protected function sortUsed(&$query) {
    // List all the terms in the specified vocabulary that are used on nodes. Order by the date the node was last changed.
    if ($this->type == ' INNER ') {
      $query->innerJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
      $query->innerJoin('node', 'n', 'ti.nid = n.nid');
    }
    elseif ($this->type = ' LEFT ') {
      $query->leftJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
      $query->leftJoin('node', 'n', 'ti.nid = n.nid');
    }
    else {
      watchdog('Suggested Terms' , 'Join type should be LEFT or INNER, but something else got passed instead.', WATCHDOG_ERROR);
    }
    $query->orderBy('n.changed', 'DESC');
    $query->orderBy('name', 'ASC');
    $query->distinct();
  }


  /**
   * Sorts a query of terms alphabetically.
   *
   * @param obj $query
   *  The query object to modify
   */
  protected function sortName(&$query) {
    // List all the term names in the specified vocabulary and order alphabetically.
    $query->groupBy('t.name');
    $query->orderBy('t.name');
    if ($this->type == ' INNER ') {
      $query->innerJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
    }
    elseif ($query_type = ' LEFT ') {
      $query->leftJoin('taxonomy_index', 'ti', 't.tid = ti.tid');
    }
    else {
      watchdog('Suggested Terms' , 'Join type should be LEFT or INNER, but something else got passed instead.', WATCHDOG_ERROR);
    }
  }
}
